@@ -1,5 +1,8 @@
 """Implements the :stac-ext:`Point Cloud Extension <pointcloud>`."""
-from typing import Any, Dict, Iterable, Generic, List, Optional, TypeVar, cast, Union
+
+from __future__ import annotations
+
+from typing import Any, Dict, Generic, Iterable, List, Optional, TypeVar, Union, cast
 
 import pystac
 from pystac.extensions.base import (
@@ -9,7 +12,7 @@ from pystac.extensions.base import (
 )
 from pystac.extensions.hooks import ExtensionHooks
 from pystac.summaries import RangeSummary
-from pystac.utils import StringEnum, map_opt, get_required
+from pystac.utils import StringEnum, get_required, map_opt
 
 T = TypeVar("T", pystac.Item, pystac.Asset)
 
@@ -70,7 +73,7 @@ class Schema:
         self.properties["type"] = type
 
     @classmethod
-    def create(cls, name: str, size: int, type: SchemaType) -> "Schema":
+    def create(cls, name: str, size: int, type: SchemaType) -> Schema:
         """Creates a new Schema.
 
         Args:
@@ -180,7 +183,7 @@ class Statistic:
         minimum: Optional[float] = None,
         stddev: Optional[float] = None,
         variance: Optional[float] = None,
-    ) -> "Statistic":
+    ) -> Statistic:
         """Creates a new Statistic class.
 
         Args:
@@ -433,7 +436,7 @@ class PointcloudExtension(
         return SCHEMA_URI
 
     @classmethod
-    def ext(cls, obj: T, add_if_missing: bool = False) -> "PointcloudExtension[T]":
+    def ext(cls, obj: T, add_if_missing: bool = False) -> PointcloudExtension[T]:
         """Extends the given STAC Object with properties from the :stac-ext:`Point Cloud
         Extension <pointcloud>`.
 
@@ -462,15 +465,15 @@ class PointcloudExtension(
     @classmethod
     def summaries(
         cls, obj: pystac.Collection, add_if_missing: bool = False
-    ) -> "SummariesPointcloudExtension":
+    ) -> SummariesPointcloudExtension:
         cls.validate_has_extension(obj, add_if_missing)
         return SummariesPointcloudExtension(obj)
 
 
 class ItemPointcloudExtension(PointcloudExtension[pystac.Item]):
-    """A concrete implementation of :class:`PointcloudExtension` on an :class:`~pystac.Item`
-    that extends the properties of the Item to include properties defined in the
-    :stac-ext:`Point Cloud Extension <pointcloud>`.
+    """A concrete implementation of :class:`PointcloudExtension` on an
+    :class:`~pystac.Item` that extends the properties of the Item to include
+    properties defined in the :stac-ext:`Point Cloud Extension <pointcloud>`.
 
     This class should generally not be instantiated directly. Instead, call
     :meth:`PointcloudExtension.ext` on an :class:`~pystac.Item` to extend it.
