@@ -1,16 +1,18 @@
 """Implements the :stac-ext:`Synthetic-Aperture Radar (SAR) Extension <sar>`."""
 
-from typing import Any, Dict, Generic, Iterable, List, Optional, TypeVar, cast, Union
+from __future__ import annotations
+
+from typing import Any, Dict, Generic, Iterable, List, Optional, TypeVar, Union, cast
 
 import pystac
-from pystac.serialization.identify import STACJSONDescription, STACVersionID
-from pystac.summaries import RangeSummary
 from pystac.extensions.base import (
     ExtensionManagementMixin,
     PropertiesExtension,
     SummariesExtension,
 )
 from pystac.extensions.hooks import ExtensionHooks
+from pystac.serialization.identify import STACJSONDescription, STACVersionID
+from pystac.summaries import RangeSummary
 from pystac.utils import StringEnum, get_required, map_opt
 
 T = TypeVar("T", pystac.Item, pystac.Asset)
@@ -298,7 +300,7 @@ class SarExtension(
         return SCHEMA_URI
 
     @classmethod
-    def ext(cls, obj: T, add_if_missing: bool = False) -> "SarExtension[T]":
+    def ext(cls, obj: T, add_if_missing: bool = False) -> SarExtension[T]:
         """Extends the given STAC Object with properties from the :stac-ext:`SAR
         Extension <sar>`.
 
@@ -327,7 +329,7 @@ class SarExtension(
     @classmethod
     def summaries(
         cls, obj: pystac.Collection, add_if_missing: bool = False
-    ) -> "SummariesSarExtension":
+    ) -> SummariesSarExtension:
         """Returns the extended summaries object for the given collection."""
         cls.validate_has_extension(obj, add_if_missing)
         return SummariesSarExtension(obj)
@@ -525,8 +527,8 @@ class SummariesSarExtension(SummariesExtension):
 
     @property
     def looks_equivalent_number(self) -> Optional[RangeSummary[float]]:
-        """Get or sets the summary of :attr:`SarExtension.looks_equivalent_number` values
-        for this Collection.
+        """Get or sets the summary of :attr:`SarExtension.looks_equivalent_number`
+        values for this Collection.
         """
 
         return self.summaries.get_range(LOOKS_EQUIVALENT_NUMBER_PROP)
