@@ -1,17 +1,15 @@
-"""Implements the Versioning Indicators extension.
+"""Implements the :stac-ext:`Versioning Indicators Extension <version>`."""
+from __future__ import annotations
 
-https://github.com/stac-extensions/version
-"""
-from enum import Enum
-from pystac.utils import get_required, map_opt
-from typing import Generic, List, Optional, TypeVar, Union, cast
+import warnings
+from contextlib import contextmanager
+from typing import Any, Dict, Generator, Generic, List, Optional, TypeVar, Union, cast
 
 import pystac
-from pystac.extensions.base import (
-    ExtensionManagementMixin,
-    PropertiesExtension,
-)
+from pystac.errors import DeprecatedWarning
+from pystac.extensions.base import ExtensionManagementMixin, PropertiesExtension
 from pystac.extensions.hooks import ExtensionHooks
+from pystac.utils import StringEnum, get_required, map_opt
 
 T = TypeVar("T", pystac.Collection, pystac.Item)
 
@@ -23,11 +21,11 @@ VERSION: str = "version"
 DEPRECATED: str = "deprecated"
 
 
-class VersionRelType(str, Enum):
+class VersionRelType(StringEnum):
     """A list of rel types defined in the Version Extension.
 
-    See the `Version Extension Relation types
-    <https://github.com/stac-extensions/version#relation-types>`__ documentation
+    See the :stac-ext:`Version Extension Relation types
+    <version#relation-types>` documentation
     for details."""
 
     LATEST = "latest-version"
@@ -117,6 +115,16 @@ class VersionExtension(
         possible and users should refrain from using it in new projects. A link with
         relation type ``latest-version`` SHOULD be added to the links and MUST refer to
         the resource that can be used instead.
+
+        NOTE:
+            A :class:`pystac.DeprecatedWarning` is issued if the ``deprecated``
+            property is ``True`` when deserializing a dictionary to an
+            :class:`~pystac.Item` or :class:`~pystac.Collection`. The
+            :meth:`~pystac.extensions.version.ignore_deprecated` context manager
+            is provided as a convenience to suppress these warnings:
+
+            >>> with ignore_deprecated():
+            ...    deprecated_item = pystac.Item.from_dict(deprecated_item_dict)
         """
         return self._get_property(DEPRECATED, bool)
 
@@ -193,7 +201,7 @@ class VersionExtension(
         return SCHEMA_URI
 
     @classmethod
-    def ext(cls, obj: T, add_if_missing: bool = False) -> "VersionExtension[T]":
+    def ext(cls, obj: T, add_if_missing: bool = False) -> VersionExtension[T]:
         """Extends the given STAC Object with properties from the :stac-ext:`Versioning
         Indicators Extension <version>`.
 
@@ -226,6 +234,10 @@ class CollectionVersionExtension(VersionExtension[pystac.Collection]):
     :meth:`VersionExtension.ext` on an :class:`~pystac.Collection` to extend it.
     """
 
+    collection: pystac.Collection
+    links: List[pystac.Link]
+    properties: Dict[str, Any]
+
     def __init__(self, collection: pystac.Collection):
         self.collection = collection
         self.properties = collection.extra_fields
@@ -245,6 +257,10 @@ class ItemVersionExtension(VersionExtension[pystac.Item]):
     :meth:`VersionExtension.ext` on an :class:`~pystac.Item` to extend it.
     """
 
+    item: pystac.Item
+    links: List[pystac.Link]
+    properties: Dict[str, Any]
+
     def __init__(self, item: pystac.Item):
         self.item = item
         self.properties = item.properties
@@ -271,3 +287,13 @@ class VersionExtensionHooks(ExtensionHooks):
 
 
 VERSION_EXTENSION_HOOKS: ExtensionHooks = VersionExtensionHooks()
+
+
+@contextmanager
+def ignore_deprecated() -> Generator[None, None, None]:
+    """Context manager for suppressing the :class:`pystac.DeprecatedWarning`
+    when creating a deprecated :class:`~pystac.Item` or :class:`~pystac.Collection`
+    from a dictionary."""
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", category=DeprecatedWarning)
+        yield
