@@ -1,28 +1,21 @@
-from abc import ABC, abstractmethod
-import os
-import json
-from typing import (
-    Any,
-    Dict,
-    List,
-    Optional,
-    TYPE_CHECKING,
-    Tuple,
-    Type,
-    Union,
-)
+from __future__ import annotations
 
-from urllib.request import urlopen
+import json
+import os
+from abc import ABC, abstractmethod
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Tuple
 from urllib.error import HTTPError
+from urllib.request import Request, urlopen
 
 import pystac
-from pystac.utils import safe_urlparse
+from pystac.link import HREF
 from pystac.serialization import (
-    merge_common_properties,
-    identify_stac_object_type,
     identify_stac_object,
+    identify_stac_object_type,
+    merge_common_properties,
     migrate_to_latest,
 )
+from pystac.utils import safe_urlparse
 
 # Use orjson if available
 try:
@@ -30,26 +23,34 @@ try:
 except ImportError:
     orjson = None  # type: ignore[assignment]
 
+# Is urllib3 available?
+try:
+    import urllib3  # noqa
+except ImportError:
+    HAS_URLLIB3 = False
+else:
+    HAS_URLLIB3 = True
+
 if TYPE_CHECKING:
-    from pystac.stac_object import STACObject as STACObject_Type
-    from pystac.catalog import Catalog as Catalog_Type
-    from pystac.link import Link as Link_Type
+    from pystac.catalog import Catalog
+    from pystac.stac_object import STACObject
 
 
 class StacIO(ABC):
-    _default_io: Optional[Type["StacIO"]] = None
+    _default_io: Optional[Callable[[], StacIO]] = None
+
+    def __init__(self, headers: Optional[Dict[str, str]] = None):
+        self.headers = headers or {}
 
     @abstractmethod
-    def read_text(
-        self, source: Union[str, "Link_Type"], *args: Any, **kwargs: Any
-    ) -> str:
+    def read_text(self, source: HREF, *args: Any, **kwargs: Any) -> str:
         """Read text from the given URI.
 
-        The source to read from can be specified as a string or a
-        :class:`~pystac.Link`. If it is a string, it must be a URI or local path from
-        which to read. Using a :class:`~pystac.Link` enables implementations to use
-        additional link information, such as paging information contained in the
-        extended links described in the `STAC API spec
+        The source to read from can be specified as a string or :class:`os.PathLike`
+        object (:class:`~pystac.Link` is a path-like object). If it is a string, it
+        must be a URI or local path from which to read. Using a :class:`~pystac.Link`
+        enables implementations to use additional link information, such as paging
+        information contained in the extended links described in the `STAC API spec
         <https://github.com/radiantearth/stac-api-spec/tree/master/item-search#paging>`__.
 
         Args:
@@ -66,14 +67,19 @@ class StacIO(ABC):
 
     @abstractmethod
     def write_text(
-        self, dest: Union[str, "Link_Type"], txt: str, *args: Any, **kwargs: Any
+        self,
+        dest: HREF,
+        txt: str,
+        *args: Any,
+        **kwargs: Any,
     ) -> None:
         """Write the given text to a file at the given URI.
 
-        The destination to write to from can be specified as a string or a
-        :class:`~pystac.Link`. If it is a string, it must be a URI or local path from
-        which to read. Using a :class:`~pystac.Link` enables implementations to use
-        additional link information.
+        The destination to write to can be specified as a string or
+        :class:`os.PathLike` object (:class:`~pystac.Link` is a path-like object). If
+        it is a string, it must be a URI or local path from which to read. Using a
+        :class:`~pystac.Link` enables implementations to use additional link
+        information.
 
         Args:
             dest : The destination to write to.
@@ -124,10 +130,10 @@ class StacIO(ABC):
     def stac_object_from_dict(
         self,
         d: Dict[str, Any],
-        href: Optional[str] = None,
-        root: Optional["Catalog_Type"] = None,
+        href: Optional[HREF] = None,
+        root: Optional[Catalog] = None,
         preserve_dict: bool = True,
-    ) -> "STACObject_Type":
+    ) -> STACObject:
         """Deserializes a :class:`~pystac.STACObject` sub-class instance from a
         dictionary.
 
@@ -143,6 +149,7 @@ class StacIO(ABC):
                 parameter. Set to ``False`` when possible to avoid the performance
                 hit of a deepcopy.
         """
+        href_str = None if href is None else str(os.fspath(href))
         if identify_stac_object_type(d) == pystac.STACObjectType.ITEM:
             collection_cache = None
             if root is not None:
@@ -150,7 +157,7 @@ class StacIO(ABC):
 
             # Merge common properties in case this is an older STAC object.
             merge_common_properties(
-                d, json_href=href, collection_cache=collection_cache
+                d, json_href=href_str, collection_cache=collection_cache
             )
 
         info = identify_stac_object(d)
@@ -158,26 +165,24 @@ class StacIO(ABC):
 
         if info.object_type == pystac.STACObjectType.CATALOG:
             result = pystac.Catalog.from_dict(
-                d, href=href, root=root, migrate=False, preserve_dict=preserve_dict
+                d, href=href_str, root=root, migrate=False, preserve_dict=preserve_dict
             )
             result._stac_io = self
             return result
 
         if info.object_type == pystac.STACObjectType.COLLECTION:
             return pystac.Collection.from_dict(
-                d, href=href, root=root, migrate=False, preserve_dict=preserve_dict
+                d, href=href_str, root=root, migrate=False, preserve_dict=preserve_dict
             )
 
         if info.object_type == pystac.STACObjectType.ITEM:
             return pystac.Item.from_dict(
-                d, href=href, root=root, migrate=False, preserve_dict=preserve_dict
+                d, href=href_str, root=root, migrate=False, preserve_dict=preserve_dict
             )
 
         raise ValueError(f"Unknown STAC object type {info.object_type}")
 
-    def read_json(
-        self, source: Union[str, "Link_Type"], *args: Any, **kwargs: Any
-    ) -> Dict[str, Any]:
+    def read_json(self, source: HREF, *args: Any, **kwargs: Any) -> Dict[str, Any]:
         """Read a dict from the given source.
 
         See :func:`StacIO.read_text <pystac.StacIO.read_text>` for usage of
@@ -199,11 +204,11 @@ class StacIO(ABC):
 
     def read_stac_object(
         self,
-        source: Union[str, "Link_Type"],
-        root: Optional["Catalog_Type"] = None,
+        source: HREF,
+        root: Optional[Catalog] = None,
         *args: Any,
         **kwargs: Any,
-    ) -> "STACObject_Type":
+    ) -> STACObject:
         """Read a STACObject from a JSON file at the given source.
 
         See :func:`StacIO.read_text <pystac.StacIO.read_text>` for usage of
@@ -224,12 +229,13 @@ class StacIO(ABC):
             contained in the file at the given uri.
         """
         d = self.read_json(source, *args, **kwargs)
-        href = source if isinstance(source, str) else source.get_absolute_href()
-        return self.stac_object_from_dict(d, href=href, root=root, preserve_dict=False)
+        return self.stac_object_from_dict(
+            d, href=source, root=root, preserve_dict=False
+        )
 
     def save_json(
         self,
-        dest: Union[str, "Link_Type"],
+        dest: HREF,
         json_dict: Dict[str, Any],
         *args: Any,
         **kwargs: Any,
@@ -251,12 +257,12 @@ class StacIO(ABC):
         self.write_text(dest, txt)
 
     @classmethod
-    def set_default(cls, stac_io_class: Type["StacIO"]) -> None:
+    def set_default(cls, stac_io_class: Callable[[], StacIO]) -> None:
         """Set the default StacIO instance to use."""
         cls._default_io = stac_io_class
 
     @classmethod
-    def default(cls) -> "StacIO":
+    def default(cls) -> StacIO:
         if cls._default_io is None:
             cls._default_io = DefaultStacIO
 
@@ -264,18 +270,12 @@ class StacIO(ABC):
 
 
 class DefaultStacIO(StacIO):
-    def read_text(self, source: Union[str, "Link_Type"], *_: Any, **__: Any) -> str:
+    def read_text(self, source: HREF, *_: Any, **__: Any) -> str:
         """A concrete implementation of :meth:`StacIO.read_text
         <pystac.StacIO.read_text>`. Converts the ``source`` argument to a string (if it
         is not already) and delegates to :meth:`DefaultStacIO.read_text_from_href` for
         opening and reading the file."""
-        href: Optional[str]
-        if isinstance(source, str):
-            href = source
-        else:
-            href = source.get_absolute_href()
-            if href is None:
-                raise IOError(f"Could not get an absolute HREF from link {source}")
+        href = str(os.fspath(source))
         return self.read_text_from_href(href)
 
     def read_text_from_href(self, href: str) -> str:
@@ -289,11 +289,11 @@ class DefaultStacIO(StacIO):
 
             href : The URI of the file to open.
         """
-        parsed = safe_urlparse(href)
         href_contents: str
-        if parsed.scheme != "":
+        if _is_url(href):
             try:
-                with urlopen(href) as f:
+                req = Request(href, headers=self.headers)
+                with urlopen(req) as f:
                     href_contents = f.read().decode("utf-8")
             except HTTPError as e:
                 raise Exception("Could not read uri {}".format(href)) from e
@@ -302,20 +302,12 @@ class DefaultStacIO(StacIO):
                 href_contents = f.read()
         return href_contents
 
-    def write_text(
-        self, dest: Union[str, "Link_Type"], txt: str, *_: Any, **__: Any
-    ) -> None:
+    def write_text(self, dest: HREF, txt: str, *_: Any, **__: Any) -> None:
         """A concrete implementation of :meth:`StacIO.write_text
         <pystac.StacIO.write_text>`. Converts the ``dest`` argument to a string (if it
         is not already) and delegates to :meth:`DefaultStacIO.write_text_from_href` for
         opening and reading the file."""
-        href: Optional[str]
-        if isinstance(dest, str):
-            href = dest
-        else:
-            href = dest.get_absolute_href()
-            if href is None:
-                raise IOError(f"Could not get an absolute HREF from link {dest}")
+        href = str(os.fspath(dest))
         return self.write_text_to_href(href, txt)
 
     def write_text_to_href(self, href: str, txt: str) -> None:
@@ -329,6 +321,7 @@ class DefaultStacIO(StacIO):
             href : The path to which the file will be written.
             txt : The string content to write to the file.
         """
+        href = os.fspath(href)
         dirname = os.path.dirname(href)
         if dirname != "" and not os.path.isdir(dirname):
             os.makedirs(dirname)
@@ -357,9 +350,7 @@ class DuplicateKeyReportingMixin(StacIO):
         )
         return result
 
-    def read_json(
-        self, source: Union[str, "Link_Type"], *args: Any, **kwargs: Any
-    ) -> Dict[str, Any]:
+    def read_json(self, source: HREF, *args: Any, **kwargs: Any) -> Dict[str, Any]:
         """Overwrites :meth:`StacIO.read_json <pystac.StacIO.read_json>` for
         deserializing a JSON file to a dictionary while checking for duplicate object
         keys.
@@ -372,7 +363,7 @@ class DuplicateKeyReportingMixin(StacIO):
         try:
             return self.json_loads(txt, source=source)
         except pystac.DuplicateObjectKeyError as e:
-            url = source if isinstance(source, str) else source.get_absolute_href()
+            url = str(os.fspath(source))
             msg = str(e) + f" in {url}"
             raise pystac.DuplicateObjectKeyError(msg)
 
@@ -389,3 +380,62 @@ class DuplicateKeyReportingMixin(StacIO):
             else:
                 result[key] = value
         return result
+
+
+def _is_url(href: str) -> bool:
+    parsed = safe_urlparse(href)
+    return parsed.scheme != ""
+
+
+if HAS_URLLIB3:
+    from typing import cast
+
+    from urllib3 import PoolManager
+    from urllib3.util import Retry
+
+    class RetryStacIO(DefaultStacIO):
+        """A customized StacIO that retries requests, using
+        :py:class:`urllib3.util.retry.Retry`.
+
+        The headers are passed to :py:class:`DefaultStacIO`. If retry is not
+        provided, a default retry is used.
+
+        To use this class, you'll need to install PySTAC with urllib3:
+
+        .. code-block:: shell
+
+            pip install pystac[urllib3]
+
+        """
+
+        retry: Retry
+        """The :py:class:`urllib3.util.retry.Retry` to use with all reading network
+        requests."""
+
+        def __init__(
+            self,
+            headers: Optional[Dict[str, str]] = None,
+            retry: Optional[Retry] = None,
+        ):
+            super().__init__(headers)
+            self.retry = retry or Retry()
+
+        def read_text_from_href(self, href: str) -> str:
+            """Reads file as a UTF-8 string, with retry support.
+
+            Args:
+                href : The URI of the file to open.
+            """
+            if _is_url(href):
+                # TODO provide a pooled StacIO to enable more efficient network
+                # access (probably named `PooledStacIO`).
+                http = PoolManager()
+                try:
+                    response = http.request(
+                        "GET", href, retries=self.retry  # type: ignore
+                    )
+                    return cast(str, response.data.decode("utf-8"))
+                except HTTPError as e:
+                    raise Exception("Could not read uri {}".format(href)) from e
+            else:
+                return super().read_text_from_href(href)
